@@ -1,12 +1,12 @@
 #ifndef CEPH_CLIENT_DIR_H
 #define CEPH_CLIENT_DIR_H
 
-class Inode;
+struct Inode;
 
 class Dir {
  public:
   Inode    *parent_inode;  // my inode
-  hash_map<string, Dentry*> dentries;
+  ceph::unordered_map<string, Dentry*> dentries;
   map<string, Dentry*> dentry_map;
   uint64_t release_count;
   uint64_t max_offset;
