@@ -38,6 +38,8 @@ enum {
 #define OSD_REP_SPLAY   1
 #define OSD_REP_CHAIN   2
 
+#define OSD_POOL_ERASURE_CODE_STRIPE_WIDTH 4096
+
 struct config_option;
 class CephContext;
 
@@ -128,7 +130,10 @@ public:
 
   // Set a configuration value.
   // Metavariables will be expanded.
-  int set_val(const char *key, const char *val, bool meta=true);
+  int set_val(const char *key, const char *val, bool meta=true, bool safe=true);
+  int set_val(const char *key, const string& s, bool meta=true, bool safe=true) {
+    return set_val(key, s.c_str(), meta, safe);
+  }
 
   // Get a configuration value.
   // No metavariables will be returned (they will have already been expanded)
@@ -173,9 +178,13 @@ private:
 
   void init_subsys();
 
-  // Expand metavariables in the provided string.
-  // Returns true if any metavariables were found and expanded.
-  bool expand_meta(std::string &val) const;
+  bool expand_meta(std::string &val,
+		   std::ostream *oss) const;
+
+  bool expand_meta(std::string &val,
+		   config_option *opt,
+		   std::list<config_option *> stack,
+		   std::ostream *oss) const;
 
   /// expand all metavariables in config structure.
   void expand_all_meta();
@@ -233,6 +242,8 @@ public:
    * It is best if this lock comes first in the lock hierarchy. We will
    * hold this lock when calling configuration observers.  */
   mutable Mutex lock;
+
+  friend class test_md_config_t;
 };
 
 typedef enum {
