@@ -29,6 +29,7 @@ class SharedLRU {
   Mutex lock;
   size_t max_size;
   Cond cond;
+  unsigned size;
 
   map<K, typename list<pair<K, VPtr> >::iterator > contents;
   list<pair<K, VPtr> > lru;
@@ -36,30 +37,36 @@ class SharedLRU {
   map<K, WeakVPtr> weak_refs;
 
   void trim_cache(list<VPtr> *to_release) {
-    while (lru.size() > max_size) {
+    while (size > max_size) {
       to_release->push_back(lru.back().second);
       lru_remove(lru.back().first);
     }
   }
 
-  void lru_remove(K key) {
-    if (!contents.count(key))
+  void lru_remove(const K& key) {
+    typename map<K, typename list<pair<K, VPtr> >::iterator>::iterator i =
+      contents.find(key);
+    if (i == contents.end())
       return;
-    lru.erase(contents[key]);
-    contents.erase(key);
+    lru.erase(i->second);
+    --size;
+    contents.erase(i);
   }
 
-  void lru_add(K key, VPtr val, list<VPtr> *to_release) {
-    if (contents.count(key)) {
-      lru.splice(lru.begin(), lru, contents[key]);
+  void lru_add(const K& key, const VPtr& val, list<VPtr> *to_release) {
+    typename map<K, typename list<pair<K, VPtr> >::iterator>::iterator i =
+      contents.find(key);
+    if (i != contents.end()) {
+      lru.splice(lru.begin(), lru, i->second);
     } else {
+      ++size;
       lru.push_front(make_pair(key, val));
       contents[key] = lru.begin();
       trim_cache(to_release);
     }
   }
 
-  void remove(K key) {
+  void remove(const K& key) {
     Mutex::Locker l(lock);
     weak_refs.erase(key);
     cond.Signal();
@@ -77,7 +84,8 @@ class SharedLRU {
   };
 
 public:
-  SharedLRU(size_t max_size = 20) : lock("SharedLRU::lock"), max_size(max_size) {}
+  SharedLRU(size_t max_size = 20)
+    : lock("SharedLRU::lock"), max_size(max_size), size(0) {}
   
   ~SharedLRU() {
     contents.clear();
@@ -85,7 +93,7 @@ public:
     assert(weak_refs.empty());
   }
 
-  void clear(K key) {
+  void clear(const K& key) {
     VPtr val; // release any ref we have after we drop the lock
     {
       Mutex::Locker l(lock);
@@ -111,7 +119,7 @@ public:
     return weak_refs.begin()->first;
   }
 
-  VPtr lower_bound(K key) {
+  VPtr lower_bound(const K& key) {
     VPtr val;
     list<VPtr> to_release;
     {
@@ -137,7 +145,7 @@ public:
     return val;
   }
 
-  VPtr lookup(K key) {
+  VPtr lookup(const K& key) {
     VPtr val;
     list<VPtr> to_release;
     {
@@ -145,8 +153,9 @@ public:
       bool retry = false;
       do {
 	retry = false;
-	if (weak_refs.count(key)) {
-	  val = weak_refs[key].lock();
+	typename map<K, WeakVPtr>::iterator i = weak_refs.find(key);
+	if (i != weak_refs.end()) {
+	  val = i->second.lock();
 	  if (val) {
 	    lru_add(key, val, &to_release);
 	  } else {
@@ -160,12 +169,35 @@ public:
     return val;
   }
 
-  VPtr add(K key, V *value) {
+  /***
+   * Inserts a key if not present, or bumps it to the front of the LRU if
+   * it is, and then gives you a reference to the value. If the key already
+   * existed, you are responsible for deleting the new value you tried to
+   * insert.
+   *
+   * @param key The key to insert
+   * @param value The value that goes with the key
+   * @param existed Set to true if the value was already in the
+   * map, false otherwise
+   * @return A reference to the map's value for the given key
+   */
+  VPtr add(const K& key, V *value, bool *existed = NULL) {
     VPtr val(value, Cleanup(this, key));
     list<VPtr> to_release;
     {
       Mutex::Locker l(lock);
-      weak_refs.insert(make_pair(key, val));
+      typename map<K, WeakVPtr>::iterator actual = weak_refs.lower_bound(key);
+      if (actual != weak_refs.end() && actual->first == key) {
+        if (existed) 
+          *existed = true;
+
+        return actual->second.lock();
+      }
+
+      if (existed)      
+        *existed = false;
+
+      weak_refs.insert(actual, make_pair(key, val));
       lru_add(key, val, &to_release);
     }
     return val;
