@@ -28,18 +28,6 @@
 
 #include <stdio.h>
 
-#ifdef DARWIN
-
-#ifndef MAP_ANON
-#define MAP_ANON 0x1000
-#endif
-#ifndef O_DIRECTORY
-#define O_DIRECTORY 0x100000
-void	*valloc(size_t);
-#endif
-
-#endif
-
 #if defined(__linux__)	// For malloc(2).
 #include <malloc.h>
 #endif
@@ -101,10 +89,30 @@ public:
   private:
     char buf[256];
   };
+  struct error_code : public malformed_input {
+    explicit error_code(int error);
+    int code;
+  };
 
 
+  /// total bytes allocated
   static int get_total_alloc();
 
+  /// enable/disable alloc tracking
+  static void track_alloc(bool b);
+
+  /// count of cached crc hits (matching input)
+  static int get_cached_crc();
+  /// count of cached crc hits (mismatching input, required adjustment)
+  static int get_cached_crc_adjusted();
+  /// enable/disable tracking of cached crcs
+  static void track_cached_crc(bool b);
+
+  /// count of calls to buffer::ptr::c_str()
+  static int get_c_str_accesses();
+  /// enable/disable tracking of buffer::ptr::c_str() calls
+  static void track_c_str(bool b);
+
 private:
  
   /* hack for memory utilization debugging. */
@@ -121,6 +129,7 @@ private:
   class raw_posix_aligned;
   class raw_hack_aligned;
   class raw_char;
+  class raw_pipe;
 
   friend std::ostream& operator<<(std::ostream& out, const raw &r);
 
@@ -136,8 +145,8 @@ public:
   static raw* claim_malloc(unsigned len, char *buf);
   static raw* create_static(unsigned len, char *buf);
   static raw* create_page_aligned(unsigned len);
-  
-  
+  static raw* create_zero_copy(unsigned len, int fd, int64_t *offset);
+
   /*
    * a buffer pointer.  references (a subsequence of) a raw buffer.
    */
@@ -194,9 +203,12 @@ public:
       memcpy(dest, c_str()+o, l);
     }
 
+    bool can_zero_copy() const;
+    int zero_copy_to_fd(int fd, int64_t *offset) const;
+
     unsigned wasted();
 
-    int cmp(const ptr& o);
+    int cmp(const ptr& o) const;
     bool is_zero() const;
 
     // modifiers
@@ -293,6 +305,7 @@ public:
 
   private:
     mutable iterator last_p;
+    int zero_copy_to_fd(int fd) const;
 
   public:
     // cons/des
@@ -330,6 +343,7 @@ public:
     }
     bool contents_equal(buffer::list& other);
 
+    bool can_zero_copy() const;
     bool is_page_aligned() const;
     bool is_n_page_sized() const;
 
@@ -367,6 +381,7 @@ public:
 
     bool is_contiguous();
     void rebuild();
+    void rebuild(ptr& nb);
     void rebuild_page_aligned();
 
     // sort-of-like-assignment-op
@@ -414,11 +429,14 @@ public:
     void encode_base64(list& o);
     void decode_base64(list& o);
 
+    void write_stream(std::ostream &out) const;
     void hexdump(std::ostream &out) const;
     int read_file(const char *fn, std::string *error);
     ssize_t read_fd(int fd, size_t len);
+    int read_fd_zero_copy(int fd, size_t len);
     int write_file(const char *fn, int mode=0644);
     int write_fd(int fd) const;
+    int write_fd_zero_copy(int fd) const;
     uint32_t crc32c(uint32_t crc) const;
   };
 
@@ -431,6 +449,7 @@ public:
 
   public:
     hash() : crc(0) { }
+    hash(uint32_t init) : crc(init) { }
 
     void update(buffer::list& bl) {
       crc = bl.crc32c(crc);
