@@ -18,8 +18,12 @@
 #include <math.h>
 #include <sys/time.h>
 #include <time.h>
+#include <errno.h>
 
 #include "include/types.h"
+#include "common/strtol.h"
+
+
 // --------
 // utime_t
 
@@ -29,10 +33,8 @@ public:
     __u32 tv_sec, tv_nsec;
   } tv;
 
-  friend class Clock;
- 
  public:
-  bool is_zero() {
+  bool is_zero() const {
     return (tv.tv_sec == 0) && (tv.tv_nsec == 0);
   }
   void normalize() {
@@ -72,6 +74,10 @@ public:
   __u32&         sec_ref()  { return tv.tv_sec; }
   __u32&         nsec_ref() { return tv.tv_nsec; }
 
+  uint64_t to_nsec() const {
+    return (uint64_t)tv.tv_nsec + (uint64_t)tv.tv_sec * 1000000000ull;
+  }
+
   void copy_to_timeval(struct timeval *v) const {
     v->tv_sec = tv.tv_sec;
     v->tv_usec = tv.tv_nsec/1000;
@@ -122,16 +128,23 @@ public:
   operator double() const {
     return (double)sec() + ((double)nsec() / 1000000000.0L);
   }
-  operator ceph_timespec() {
+  operator ceph_timespec() const {
     ceph_timespec ts;
     ts.tv_sec = sec();
     ts.tv_nsec = nsec();
     return ts;
   }
 
+  void sleep() const {
+    struct timespec ts;
+    to_timespec(&ts);
+    nanosleep(&ts, NULL);
+  }
+
   // output
   ostream& gmtime(ostream& out) const {
     out.setf(std::ios::right);
+    char oldfill = out.fill();
     out.fill('0');
     if (sec() < ((time_t)(60*60*24*365*10))) {
       // raw seconds.  this looks like a relative time.
@@ -152,12 +165,41 @@ public:
       out << "." << std::setw(6) << usec();
       out << "Z";
     }
+    out.fill(oldfill);
     out.unsetf(std::ios::right);
     return out;
   }
 
+  // output
+  ostream& asctime(ostream& out) const {
+    out.setf(std::ios::right);
+    char oldfill = out.fill();
+    out.fill('0');
+    if (sec() < ((time_t)(60*60*24*365*10))) {
+      // raw seconds.  this looks like a relative time.
+      out << (long)sec() << "." << std::setw(6) << usec();
+    } else {
+      // localtime.  this looks like an absolute time.
+      //  aim for http://en.wikipedia.org/wiki/ISO_8601
+      struct tm bdt;
+      time_t tt = sec();
+      gmtime_r(&tt, &bdt);
+
+      char buf[128];
+      asctime_r(&bdt, buf);
+      int len = strlen(buf);
+      if (buf[len - 1] == '\n')
+        buf[len - 1] = '\0';
+      out << buf;
+    }
+    out.fill(oldfill);
+    out.unsetf(std::ios::right);
+    return out;
+  }
+  
   ostream& localtime(ostream& out) const {
     out.setf(std::ios::right);
+    char oldfill = out.fill();
     out.fill('0');
     if (sec() < ((time_t)(60*60*24*365*10))) {
       // raw seconds.  this looks like a relative time.
@@ -178,6 +220,7 @@ public:
       out << "." << std::setw(6) << usec();
       //out << '_' << bdt.tm_zone;
     }
+    out.fill(oldfill);
     out.unsetf(std::ios::right);
     return out;
   }
@@ -192,6 +235,60 @@ public:
 		    bdt.tm_year + 1900, bdt.tm_mon + 1, bdt.tm_mday,
 		    bdt.tm_hour, bdt.tm_min, bdt.tm_sec, usec());
   }
+
+  static int parse_date(const string& date, uint64_t *epoch, uint64_t *nsec,
+                        string *out_date=NULL, string *out_time=NULL) {
+    struct tm tm;
+    memset(&tm, 0, sizeof(tm));
+
+    if (nsec)
+      *nsec = 0;
+
+    const char *p = strptime(date.c_str(), "%Y-%m-%d", &tm);
+    if (p) {
+      if (*p == ' ') {
+	p++;
+	p = strptime(p, " %H:%M:%S", &tm);
+	if (!p)
+	  return -EINVAL;
+        if (nsec && *p == '.') {
+          ++p;
+          unsigned i;
+          char buf[10]; /* 9 digit + null termination */
+          for (i = 0; (i < sizeof(buf) - 1) && isdigit(*p); ++i, ++p) {
+            buf[i] = *p;
+          }
+          for (; i < sizeof(buf) - 1; ++i) {
+            buf[i] = '0';
+          }
+          buf[i] = '\0';
+          string err;
+          *nsec = (uint64_t)strict_strtol(buf, 10, &err);
+          if (!err.empty()) {
+            return -EINVAL;
+          }
+        }
+      }
+    } else {
+      return -EINVAL;
+    }
+    time_t t = timegm(&tm);
+    if (epoch)
+      *epoch = (uint64_t)t;
+
+    if (out_date) {
+      char buf[32];
+      strftime(buf, sizeof(buf), "%F", &tm);
+      *out_date = buf;
+    }
+    if (out_time) {
+      char buf[32];
+      strftime(buf, sizeof(buf), "%T", &tm);
+      *out_time = buf;
+    }
+
+    return 0;
+  }
 };
 WRITE_CLASS_ENCODER(utime_t)
 
