@@ -30,7 +30,7 @@
 
 #include "messages/MWatchNotify.h"
 #include "messages/MLog.h"
-#include "msg/Messenger.h"
+#include "msg/SimpleMessenger.h"
 
 // needed for static_cast
 #include "messages/PaxosServiceMessage.h"
@@ -52,6 +52,8 @@
 #undef dout_prefix
 #define dout_prefix *_dout << "librados: "
 
+static atomic_t rados_instance;
+
 bool librados::RadosClient::ms_get_authorizer(int dest_type,
 					      AuthAuthorizer **authorizer,
 					      bool force_new) {
@@ -204,6 +206,7 @@ int librados::RadosClient::connect()
   common_init_finish(cct);
 
   int err;
+  uint64_t nonce;
 
   // already connected?
   if (state == CONNECTING)
@@ -218,7 +221,8 @@ int librados::RadosClient::connect()
     goto out;
 
   err = -ENOMEM;
-  messenger = Messenger::create_client_messenger(cct, "radosclient");
+  nonce = getpid() + (1000000 * (uint64_t)rados_instance.inc());
+  messenger = new SimpleMessenger(cct, entity_name_t::CLIENT(-1), "radosclient", nonce);
   if (!messenger)
     goto out;
 
@@ -232,7 +236,7 @@ int librados::RadosClient::connect()
   ldout(cct, 1) << "starting objecter" << dendl;
 
   err = -ENOMEM;
-  objecter = new (std::nothrow) Objecter(cct, messenger, &monclient, &osdmap, lock, timer,
+  objecter = new Objecter(cct, messenger, &monclient, &osdmap, lock, timer,
 			  cct->_conf->rados_mon_op_timeout,
 			  cct->_conf->rados_osd_op_timeout);
   if (!objecter)
@@ -283,19 +287,8 @@ int librados::RadosClient::connect()
   err = 0;
 
  out:
-  if (err) {
+  if (err)
     state = DISCONNECTED;
-
-    if (objecter) {
-      delete objecter;
-      objecter = NULL;
-    }
-    if (messenger) {
-      delete messenger;
-      messenger = NULL;
-    }
-  }
-
   return err;
 }
 
@@ -441,10 +434,6 @@ int librados::RadosClient::wait_for_osdmap()
 {
   assert(lock.is_locked());
 
-  if (state != CONNECTED) {
-    return -ENOTCONN;
-  }
-
   utime_t timeout;
   if (cct->_conf->rados_mon_op_timeout > 0)
     timeout.set_from_double(cct->_conf->rados_mon_op_timeout);
@@ -557,13 +546,19 @@ bool librados::RadosClient::put() {
 int librados::RadosClient::pool_create(string& name, unsigned long long auid,
 				       __u8 crush_rule)
 {
-  int reply;
+  lock.Lock();
+
+  int r = wait_for_osdmap();
+  if (r < 0) {
+    lock.Unlock();
+    return r;
+  }
 
   Mutex mylock ("RadosClient::pool_create::mylock");
+  int reply;
   Cond cond;
   bool done;
   Context *onfinish = new C_SafeCond(&mylock, &cond, &done, &reply);
-  lock.Lock();
   reply = objecter->create_pool(name, onfinish, auid, crush_rule);
   lock.Unlock();
 
@@ -583,8 +578,13 @@ int librados::RadosClient::pool_create_async(string& name, PoolAsyncCompletionIm
 					     __u8 crush_rule)
 {
   Mutex::Locker l(lock);
+
+  int r = wait_for_osdmap();
+  if (r < 0)
+    return r;
+
   Context *onfinish = new C_PoolAsync_Safe(c);
-  int r = objecter->create_pool(name, onfinish, auid, crush_rule);
+  r = objecter->create_pool(name, onfinish, auid, crush_rule);
   if (r < 0) {
     delete onfinish;
   }
