@@ -29,8 +29,6 @@ class Session;
 class CDir;
 class CInode;
 class CDentry;
-struct Mutation;
-struct MDRequest;
 class EMetaBlob;
 struct SnapRealm;
 
@@ -45,7 +43,6 @@ class MLock;
 
 class MClientRequest;
 
-class Anchor;
 class Capability;
 class LogSegment;
 
@@ -77,25 +74,26 @@ protected:
   void send_lock_message(SimpleLock *lock, int msg, const bufferlist &data);
 
   // -- locks --
-  void _drop_rdlocks(Mutation *mut, set<CInode*> *pneed_issue);
-  void _drop_non_rdlocks(Mutation *mut, set<CInode*> *pneed_issue);
+  void _drop_rdlocks(MutationImpl *mut, set<CInode*> *pneed_issue);
+  void _drop_non_rdlocks(MutationImpl *mut, set<CInode*> *pneed_issue);
 public:
   void include_snap_rdlocks(set<SimpleLock*>& rdlocks, CInode *in);
   void include_snap_rdlocks_wlayout(set<SimpleLock*>& rdlocks, CInode *in,
                                     ceph_file_layout **layout);
 
-  bool acquire_locks(MDRequest *mdr,
+  bool acquire_locks(MDRequestRef& mdr,
 		     set<SimpleLock*> &rdlocks,
 		     set<SimpleLock*> &wrlocks,
 		     set<SimpleLock*> &xlocks,
 		     map<SimpleLock*,int> *remote_wrlocks=NULL,
-		     CInode *auth_pin_freeze=NULL);
+		     CInode *auth_pin_freeze=NULL,
+		     bool auth_pin_nonblock=false);
 
-  void cancel_locking(Mutation *mut, set<CInode*> *pneed_issue);
-  void drop_locks(Mutation *mut, set<CInode*> *pneed_issue=0);
-  void set_xlocks_done(Mutation *mut, bool skip_dentry=false);
-  void drop_non_rdlocks(Mutation *mut, set<CInode*> *pneed_issue=0);
-  void drop_rdlocks(Mutation *mut, set<CInode*> *pneed_issue=0);
+  void cancel_locking(MutationImpl *mut, set<CInode*> *pneed_issue);
+  void drop_locks(MutationImpl *mut, set<CInode*> *pneed_issue=0);
+  void set_xlocks_done(MutationImpl *mut, bool skip_dentry=false);
+  void drop_non_rdlocks(MutationImpl *mut, set<CInode*> *pneed_issue=0);
+  void drop_rdlocks(MutationImpl *mut, set<CInode*> *pneed_issue=0);
 
   void eval_gather(SimpleLock *lock, bool first=false, bool *need_issue=0, list<Context*> *pfinishers=0);
   void eval(SimpleLock *lock, bool *need_issue);
@@ -128,26 +126,25 @@ public:
 
   bool _rdlock_kick(SimpleLock *lock, bool as_anon);
   bool rdlock_try(SimpleLock *lock, client_t client, Context *c);
-  bool rdlock_start(SimpleLock *lock, MDRequest *mut, bool as_anon=false);
-  void rdlock_finish(SimpleLock *lock, Mutation *mut, bool *pneed_issue);
+  bool rdlock_start(SimpleLock *lock, MDRequestRef& mut, bool as_anon=false);
+  void rdlock_finish(SimpleLock *lock, MutationImpl *mut, bool *pneed_issue);
   bool can_rdlock_set(set<SimpleLock*>& locks);
   bool rdlock_try_set(set<SimpleLock*>& locks);
-  void rdlock_take_set(set<SimpleLock*>& locks);
-  void rdlock_finish_set(set<SimpleLock*>& locks);
+  void rdlock_take_set(set<SimpleLock*>& locks, MutationRef& mut);
 
-  void wrlock_force(SimpleLock *lock, Mutation *mut);
-  bool wrlock_start(SimpleLock *lock, MDRequest *mut, bool nowait=false);
-  void wrlock_finish(SimpleLock *lock, Mutation *mut, bool *pneed_issue);
+  void wrlock_force(SimpleLock *lock, MutationRef& mut);
+  bool wrlock_start(SimpleLock *lock, MDRequestRef& mut, bool nowait=false);
+  void wrlock_finish(SimpleLock *lock, MutationImpl *mut, bool *pneed_issue);
 
-  void remote_wrlock_start(SimpleLock *lock, int target, MDRequest *mut);
-  void remote_wrlock_finish(SimpleLock *lock, int target, Mutation *mut);
+  void remote_wrlock_start(SimpleLock *lock, int target, MDRequestRef& mut);
+  void remote_wrlock_finish(SimpleLock *lock, int target, MutationImpl *mut);
 
-  bool xlock_start(SimpleLock *lock, MDRequest *mut);
+  bool xlock_start(SimpleLock *lock, MDRequestRef& mut);
   void _finish_xlock(SimpleLock *lock, client_t xlocker, bool *pneed_issue);
-  void xlock_finish(SimpleLock *lock, Mutation *mut, bool *pneed_issue);
+  void xlock_finish(SimpleLock *lock, MutationImpl *mut, bool *pneed_issue);
 
-  void xlock_export(SimpleLock *lock, Mutation *mut);
-  void xlock_import(SimpleLock *lock, Mutation *mut);
+  void xlock_export(SimpleLock *lock, MutationImpl *mut);
+  void xlock_import(SimpleLock *lock);
 
 
   // simple
@@ -184,14 +181,15 @@ protected:
   class C_Locker_ScatterWB : public Context {
     Locker *locker;
     ScatterLock *lock;
-    Mutation *mut;
+    MutationRef mut;
   public:
-    C_Locker_ScatterWB(Locker *l, ScatterLock *sl, Mutation *m) : locker(l), lock(sl), mut(m) {}
+    C_Locker_ScatterWB(Locker *l, ScatterLock *sl, MutationRef& m) :
+      locker(l), lock(sl), mut(m) {}
     void finish(int r) { 
       locker->scatter_writebehind_finish(lock, mut); 
     }
   };
-  void scatter_writebehind_finish(ScatterLock *lock, Mutation *mut);
+  void scatter_writebehind_finish(ScatterLock *lock, MutationRef& mut);
 
   xlist<ScatterLock*> updated_scatterlocks;
 public:
@@ -209,10 +207,11 @@ public:
   // process_request_cap_release to preserve ordering.
   bool should_defer_client_cap_frozen(CInode *in);
 
-  void process_request_cap_release(MDRequest *mdr, client_t client, const ceph_mds_request_release& r,
+  void process_request_cap_release(MDRequestRef& mdr, client_t client, const ceph_mds_request_release& r,
 				   const string &dname);
 
-  void kick_cap_releases(MDRequest *mdr);
+  void kick_cap_releases(MDRequestRef& mdr);
+  void kick_issue_caps(CInode *in, client_t client, ceph_seq_t seq);
 
   void remove_client_cap(CInode *in, client_t client);
 
@@ -230,12 +229,12 @@ public:
 
   // local
 public:
-  void local_wrlock_grab(LocalLock *lock, Mutation *mut);
+  void local_wrlock_grab(LocalLock *lock, MutationRef& mut);
 protected:
-  bool local_wrlock_start(LocalLock *lock, MDRequest *mut);
-  void local_wrlock_finish(LocalLock *lock, Mutation *mut);
-  bool local_xlock_start(LocalLock *lock, MDRequest *mut);
-  void local_xlock_finish(LocalLock *lock, Mutation *mut);
+  bool local_wrlock_start(LocalLock *lock, MDRequestRef& mut);
+  void local_wrlock_finish(LocalLock *lock, MutationImpl *mut);
+  bool local_xlock_start(LocalLock *lock, MDRequestRef& mut);
+  void local_xlock_finish(LocalLock *lock, MutationImpl *mut);
 
 
   // file
@@ -271,7 +270,7 @@ public:
 protected:
   void handle_inode_file_caps(class MInodeFileCaps *m);
 
-  void file_update_finish(CInode *in, Mutation *mut, bool share, client_t client, Capability *cap,
+  void file_update_finish(CInode *in, MutationRef& mut, bool share, client_t client, Capability *cap,
 			  MClientCaps *ack);
 public:
   void calc_new_client_ranges(CInode *in, uint64_t size, map<client_t, client_writeable_range_t>& new_ranges);
