@@ -78,15 +78,8 @@ void Elector::start()
   init();
   
   // start by trying to elect me
-  if (epoch % 2 == 0) {
+  if (epoch % 2 == 0) 
     bump_epoch(epoch+1);  // odd == election cycle
-  } else {
-    // do a trivial db write just to ensure it is writeable.
-    MonitorDBStore::Transaction t;
-    t.put(Monitor::MONITOR_NAME, "election_writeable_test", rand());
-    int r = mon->store->apply_transaction(t);
-    assert(r >= 0);
-  }
   start_stamp = ceph_clock_now(g_ceph_context);
   electing_me = true;
   acked_me[mon->rank] = CEPH_FEATURES_ALL;
@@ -372,7 +365,7 @@ void Elector::nak_old_peer(MMonElection *m)
 					   mon->monmap);
     reply->quorum_features = required_features;
     mon->features.encode(reply->sharing_bl);
-    mon->messenger->send_message(reply, m->get_connection());
+    m->get_connection()->send_message(reply);
   }
   m->put();
 }
