@@ -21,7 +21,6 @@
 #include <boost/scoped_ptr.hpp>
 #include <sstream>
 #include "os/KeyValueDB.h"
-#include "os/LevelDBStore.h"
 
 #include "include/assert.h"
 #include "common/Formatter.h"
@@ -29,7 +28,7 @@
 
 class MonitorDBStore
 {
-  boost::scoped_ptr<LevelDBStore> db;
+  boost::scoped_ptr<KeyValueDB> db;
   bool do_dump;
   int dump_fd;
 
@@ -87,6 +86,9 @@ class MonitorDBStore
 
   struct Transaction {
     list<Op> ops;
+    uint64_t bytes, keys;
+
+    Transaction() : bytes(0), keys(0) {}
 
     enum {
       OP_PUT	= 1,
@@ -96,6 +98,8 @@ class MonitorDBStore
 
     void put(string prefix, string key, bufferlist& bl) {
       ops.push_back(Op(OP_PUT, prefix, key, bl));
+      ++keys;
+      bytes += prefix.length() + key.length() + bl.length();
     }
 
     void put(string prefix, version_t ver, bufferlist& bl) {
@@ -112,6 +116,8 @@ class MonitorDBStore
 
     void erase(string prefix, string key) {
       ops.push_back(Op(OP_ERASE, prefix, key));
+      ++keys;
+      bytes += prefix.length() + key.length();
     }
 
     void erase(string prefix, version_t ver) {
@@ -129,14 +135,20 @@ class MonitorDBStore
     }
 
     void encode(bufferlist& bl) const {
-      ENCODE_START(1, 1, bl);
+      ENCODE_START(2, 1, bl);
       ::encode(ops, bl);
+      ::encode(bytes, bl);
+      ::encode(keys, bl);
       ENCODE_FINISH(bl);
     }
 
     void decode(bufferlist::iterator& bl) {
-      DECODE_START(1, bl);
+      DECODE_START(2, bl);
       ::decode(ops, bl);
+      if (struct_v >= 2) {
+	::decode(bytes, bl);
+	::decode(keys, bl);
+      }
       DECODE_FINISH(bl);
     }
 
@@ -153,6 +165,8 @@ class MonitorDBStore
 
     void append(Transaction& other) {
       ops.splice(ops.end(), other.ops);
+      keys += other.keys;
+      bytes += other.bytes;
     }
 
     void append_from_encoded(bufferlist& bl) {
@@ -169,6 +183,12 @@ class MonitorDBStore
     bool size() {
       return ops.size();
     }
+    uint64_t get_keys() const {
+      return keys;
+    }
+    uint64_t get_bytes() const {
+      return bytes;
+    }
 
     void dump(ceph::Formatter *f, bool dump_val=false) const {
       f->open_object_section("transaction");
@@ -218,6 +238,8 @@ class MonitorDBStore
 	f->close_section();
       }
       f->close_section();
+      f->dump_unsigned("num_keys", keys);
+      f->dump_unsigned("num_bytes", bytes);
       f->close_section();
     }
   };
@@ -486,33 +508,13 @@ class MonitorDBStore
     db->submit_transaction_sync(dbt);
   }
 
-  void init_options() {
-    db->init();
-    if (g_conf->mon_leveldb_write_buffer_size)
-      db->options.write_buffer_size = g_conf->mon_leveldb_write_buffer_size;
-    if (g_conf->mon_leveldb_cache_size)
-      db->options.cache_size = g_conf->mon_leveldb_cache_size;
-    if (g_conf->mon_leveldb_block_size)
-      db->options.block_size = g_conf->mon_leveldb_block_size;
-    if (g_conf->mon_leveldb_bloom_size)
-      db->options.bloom_size = g_conf->mon_leveldb_bloom_size;
-    if (g_conf->mon_leveldb_compression)
-      db->options.compression_enabled = g_conf->mon_leveldb_compression;
-    if (g_conf->mon_leveldb_max_open_files)
-      db->options.max_open_files = g_conf->mon_leveldb_max_open_files;
-    if (g_conf->mon_leveldb_paranoid)
-      db->options.paranoid_checks = g_conf->mon_leveldb_paranoid;
-    if (g_conf->mon_leveldb_log.length())
-      db->options.log_file = g_conf->mon_leveldb_log;
-  }
-
   int open(ostream &out) {
-    init_options();
+    db->init();
     return db->open(out);
   }
 
   int create_and_open(ostream &out) {
-    init_options();
+    db->init();
     return db->create_and_open(out);
   }
 
@@ -540,11 +542,14 @@ class MonitorDBStore
     os << path.substr(0, path.size() - pos) << "/store.db";
     string full_path = os.str();
 
-    LevelDBStore *db_ptr = new LevelDBStore(g_ceph_context, full_path);
+    KeyValueDB *db_ptr = KeyValueDB::create(g_ceph_context,
+					    g_conf->mon_keyvaluedb,
+					    full_path);
     if (!db_ptr) {
-      derr << __func__ << " error initializing level db back storage in "
-		<< full_path << dendl;
-      assert(0 != "MonitorDBStore: error initializing level db back storage");
+      derr << __func__ << " error initializing "
+	   << g_conf->mon_keyvaluedb << " db back storage in "
+	   << full_path << dendl;
+      assert(0 != "MonitorDBStore: error initializing keyvaluedb back storage");
     }
     db.reset(db_ptr);
 
@@ -560,7 +565,7 @@ class MonitorDBStore
       }
     }
   }
-  MonitorDBStore(LevelDBStore *db_ptr) :
+  MonitorDBStore(KeyValueDB *db_ptr) :
     db(0), do_dump(false), dump_fd(-1) {
     db.reset(db_ptr);
   }
