@@ -87,9 +87,11 @@ void PGMonitor::update_logger()
 {
   dout(10) << "update_logger" << dendl;
 
-  mon->cluster_logger->set(l_cluster_osd_kb, pg_map.osd_sum.kb);
-  mon->cluster_logger->set(l_cluster_osd_kb_used, pg_map.osd_sum.kb_used);
-  mon->cluster_logger->set(l_cluster_osd_kb_avail, pg_map.osd_sum.kb_avail);
+  mon->cluster_logger->set(l_cluster_osd_bytes, pg_map.osd_sum.kb * 1024ull);
+  mon->cluster_logger->set(l_cluster_osd_bytes_used,
+			   pg_map.osd_sum.kb_used * 1024ull);
+  mon->cluster_logger->set(l_cluster_osd_bytes_avail,
+			   pg_map.osd_sum.kb_avail * 1024ull);
 
   mon->cluster_logger->set(l_cluster_num_pool, pg_map.pg_pool_sum.size());
   mon->cluster_logger->set(l_cluster_num_pg, pg_map.pg_stat.size());
@@ -112,6 +114,7 @@ void PGMonitor::update_logger()
 
   mon->cluster_logger->set(l_cluster_num_object, pg_map.pg_sum.stats.sum.num_objects);
   mon->cluster_logger->set(l_cluster_num_object_degraded, pg_map.pg_sum.stats.sum.num_objects_degraded);
+  mon->cluster_logger->set(l_cluster_num_object_misplaced, pg_map.pg_sum.stats.sum.num_objects_misplaced);
   mon->cluster_logger->set(l_cluster_num_object_unfound, pg_map.pg_sum.stats.sum.num_objects_unfound);
   mon->cluster_logger->set(l_cluster_num_bytes, pg_map.pg_sum.stats.sum.num_bytes);
 }
@@ -1168,7 +1171,7 @@ void PGMonitor::send_pg_creates(int osd, Connection *con)
   }
 
   if (con) {
-    mon->messenger->send_message(m, con);
+    con->send_message(m);
   } else {
     assert(mon->osdmon()->osdmap.is_up(osd));
     mon->messenger->send_message(m, mon->osdmon()->osdmap.get_inst(osd));
@@ -1231,9 +1234,9 @@ void PGMonitor::dump_object_stat_sum(TextTable &tbl, Formatter *f,
     if (verbose) {
       f->dump_int("dirty", sum.num_objects_dirty);
       f->dump_int("rd", sum.num_rd);
-      f->dump_int("rd_kb", sum.num_rd_kb);
+      f->dump_int("rd_bytes", sum.num_rd_kb * 1024ull);
       f->dump_int("wr", sum.num_wr);
-      f->dump_int("wr_kb", sum.num_wr_kb);
+      f->dump_int("wr_bytes", sum.num_wr_kb * 1024ull);
     }
   } else {
     tbl << stringify(si_t(sum.num_bytes));
@@ -1284,7 +1287,7 @@ void PGMonitor::dump_pool_stats(stringstream &ss, Formatter *f, bool verbose)
     if (verbose)
       tbl.define_column("CATEGORY", TextTable::LEFT, TextTable::LEFT);
     tbl.define_column("USED", TextTable::LEFT, TextTable::RIGHT);
-    tbl.define_column("%%USED", TextTable::LEFT, TextTable::RIGHT);
+    tbl.define_column("%USED", TextTable::LEFT, TextTable::RIGHT);
     tbl.define_column("MAX AVAIL", TextTable::LEFT, TextTable::RIGHT);
     tbl.define_column("OBJECTS", TextTable::LEFT, TextTable::RIGHT);
     if (verbose) {
@@ -1391,9 +1394,9 @@ void PGMonitor::dump_fs_stats(stringstream &ss, Formatter *f, bool verbose)
 {
   if (f) {
     f->open_object_section("stats");
-    f->dump_int("total_space", pg_map.osd_sum.kb);
-    f->dump_int("total_used", pg_map.osd_sum.kb_used);
-    f->dump_int("total_avail", pg_map.osd_sum.kb_avail);
+    f->dump_int("total_bytes", pg_map.osd_sum.kb * 1024ull);
+    f->dump_int("total_used_bytes", pg_map.osd_sum.kb_used * 1024ull);
+    f->dump_int("total_avail_bytes", pg_map.osd_sum.kb_avail * 1024ull);
     if (verbose) {
       f->dump_int("total_objects", pg_map.pg_sum.stats.sum.num_objects);
     }
@@ -1403,7 +1406,7 @@ void PGMonitor::dump_fs_stats(stringstream &ss, Formatter *f, bool verbose)
     tbl.define_column("SIZE", TextTable::LEFT, TextTable::RIGHT);
     tbl.define_column("AVAIL", TextTable::LEFT, TextTable::RIGHT);
     tbl.define_column("RAW USED", TextTable::LEFT, TextTable::RIGHT);
-    tbl.define_column("%%RAW USED", TextTable::LEFT, TextTable::RIGHT);
+    tbl.define_column("%RAW USED", TextTable::LEFT, TextTable::RIGHT);
     if (verbose) {
       tbl.define_column("OBJECTS", TextTable::LEFT, TextTable::RIGHT);
     }
@@ -1787,6 +1790,14 @@ static void note_stuck_detail(enum PGMap::StuckPG what,
       since = p->second.last_clean;
       whatname = "unclean";
       break;
+    case PGMap::STUCK_DEGRADED:
+      since = p->second.last_undegraded;
+      whatname = "degraded";
+      break;
+    case PGMap::STUCK_UNDERSIZED:
+      since = p->second.last_fullsized;
+      whatname = "undersized";
+      break;
     case PGMap::STUCK_STALE:
       since = p->second.last_unstale;
       whatname = "stale";
@@ -1838,6 +1849,8 @@ void PGMonitor::get_health(list<pair<health_status_t,string> >& summary,
       note["stale"] += p->second;
     if (p->first & PG_STATE_DOWN)
       note["down"] += p->second;
+    if (p->first & PG_STATE_UNDERSIZED)
+      note["undersized"] += p->second;
     if (p->first & PG_STATE_DEGRADED)
       note["degraded"] += p->second;
     if (p->first & PG_STATE_INCONSISTENT)
@@ -1882,6 +1895,22 @@ void PGMonitor::get_health(list<pair<health_status_t,string> >& summary,
   }
   stuck_pgs.clear();
 
+  pg_map.get_stuck_stats(PGMap::STUCK_UNDERSIZED, cutoff, stuck_pgs);
+  if (!stuck_pgs.empty()) {
+    note["stuck undersized"] = stuck_pgs.size();
+    if (detail)
+      note_stuck_detail(PGMap::STUCK_UNDERSIZED, stuck_pgs, detail);
+  }
+  stuck_pgs.clear();
+
+  pg_map.get_stuck_stats(PGMap::STUCK_DEGRADED, cutoff, stuck_pgs);
+  if (!stuck_pgs.empty()) {
+    note["stuck degraded"] = stuck_pgs.size();
+    if (detail)
+      note_stuck_detail(PGMap::STUCK_DEGRADED, stuck_pgs, detail);
+  }
+  stuck_pgs.clear();
+
   pg_map.get_stuck_stats(PGMap::STUCK_STALE, cutoff, stuck_pgs);
   if (!stuck_pgs.empty()) {
     note["stuck stale"] = stuck_pgs.size();
@@ -1901,6 +1930,7 @@ void PGMonitor::get_health(list<pair<health_status_t,string> >& summary,
 	   ++p) {
 	if ((p->second.state & (PG_STATE_STALE |
 			       PG_STATE_DOWN |
+			       PG_STATE_UNDERSIZED |
 			       PG_STATE_DEGRADED |
 			       PG_STATE_INCONSISTENT |
 			       PG_STATE_PEERING |
@@ -2110,6 +2140,10 @@ int PGMonitor::dump_stuck_pg_stats(stringstream &ds,
     stuck_type = PGMap::STUCK_INACTIVE;
   else if (type == "unclean")
     stuck_type = PGMap::STUCK_UNCLEAN;
+  else if (type == "undersized")
+    stuck_type = PGMap::STUCK_UNDERSIZED;
+  else if (type == "degraded")
+    stuck_type = PGMap::STUCK_DEGRADED;
   else if (type == "stale")
     stuck_type = PGMap::STUCK_STALE;
   else {
