@@ -282,7 +282,7 @@ void Paxos::handle_collect(MMonPaxos *collect)
   }
 
   // send reply
-  mon->messenger->send_message(last, collect->get_source_inst());
+  collect->get_connection()->send_message(last);
   collect->put();
 }
 
@@ -464,7 +464,7 @@ void Paxos::handle_last(MMonPaxos *last)
   peer_first_committed[from] = last->first_committed;
   peer_last_committed[from] = last->last_committed;
 
-  if (last->first_committed > last_committed + 1) {
+  if (last->first_committed > last_committed+1) {
     dout(5) << __func__
             << " mon." << from
 	    << " lowest version is too high for our last committed"
@@ -486,7 +486,7 @@ void Paxos::handle_last(MMonPaxos *last)
   for (map<int,version_t>::iterator p = peer_last_committed.begin();
        p != peer_last_committed.end();
        ++p) {
-    if (p->second + 1 < first_committed && first_committed > 1) {
+    if (p->second < first_committed && first_committed > 1) {
       dout(5) << __func__
 	      << " peon " << p->first
 	      << " last_committed (" << p->second
@@ -753,7 +753,7 @@ void Paxos::handle_begin(MMonPaxos *begin)
 				    ceph_clock_now(g_ceph_context));
   accept->pn = accepted_pn;
   accept->last_committed = last_committed;
-  mon->messenger->send_message(accept, begin->get_source_inst());
+  begin->get_connection()->send_message(accept);
   
   begin->put();
 }
@@ -1076,7 +1076,7 @@ void Paxos::handle_lease(MMonPaxos *lease)
   ack->last_committed = last_committed;
   ack->first_committed = first_committed;
   ack->lease_timestamp = ceph_clock_now(g_ceph_context);
-  mon->messenger->send_message(ack, lease->get_source_inst());
+  lease->get_connection()->send_message(ack);
 
   // (re)set timeout event.
   reset_lease_timeout();
@@ -1389,8 +1389,7 @@ void Paxos::dispatch(PaxosServiceMessage *m)
 
 bool Paxos::is_readable(version_t v)
 {
-  dout(5) << "is_readable now=" << ceph_clock_now(g_ceph_context)
-	  << " lease_expire=" << lease_expire
+  dout(1) << "is_readable now=" << ceph_clock_now(g_ceph_context) << " lease_expire=" << lease_expire
 	  << " has v" << v << " lc " << last_committed << dendl;
   if (v > last_committed)
     return false;
