@@ -72,8 +72,10 @@ ostream& Pipe::_pipe_prefix(std::ostream *_dout) {
  * Pipe
  */
 
-Pipe::Pipe(SimpleMessenger *r, int st, Connection *con)
-  : reader_thread(this), writer_thread(this),
+Pipe::Pipe(SimpleMessenger *r, int st, PipeConnection *con)
+  : RefCountedObject(r->cct),
+    reader_thread(this),
+    writer_thread(this),
     delay_thread(NULL),
     msgr(r),
     conn_id(r->dispatch_queue.get_id()),
@@ -88,14 +90,13 @@ Pipe::Pipe(SimpleMessenger *r, int st, Connection *con)
     in_q(&(r->dispatch_queue)),
     send_keepalive(false),
     send_keepalive_ack(false),
-    close_on_empty(false),
     connect_seq(0), peer_global_seq(0),
     out_seq(0), in_seq(0), in_seq_acked(0) {
   if (con) {
     connection_state = con;
     connection_state->reset_pipe(this);
   } else {
-    connection_state = new Connection(msgr);
+    connection_state = new PipeConnection(msgr->cct, msgr);
     connection_state->pipe = get();
   }
 
@@ -128,11 +129,6 @@ void Pipe::handle_ack(uint64_t seq)
 				<< seq << " >= " << m->get_seq() << " on " << m << " " << *m << dendl;
     m->put();
   }
-
-  if (sent.empty() && close_on_empty) {
-    lsubdout(msgr->cct, ms, 10) << "reader got last ack, queue empty, closing" << dendl;
-    stop();
-  }
 }
 
 void Pipe::start_reader()
@@ -937,7 +933,7 @@ int Pipe::connect()
 
     ceph_msg_connect connect;
     connect.features = policy.features_supported;
-    connect.host_type = msgr->my_type;
+    connect.host_type = msgr->get_myinst().name.type();
     connect.global_seq = gseq;
     connect.connect_seq = cseq;
     connect.protocol_version = msgr->get_proto_version(peer_type, true);
@@ -1407,8 +1403,14 @@ void Pipe::stop_and_wait()
   if (state != STATE_CLOSED)
     stop();
   
+  // HACK: we work around an annoying deadlock here.  If the fast
+  // dispatch method calls mark_down() on itself, it can block here
+  // waiting for the reader_dispatching flag to clear... which will
+  // clearly never happen.  Avoid the situation by skipping the wait
+  // if we are marking our *own* connect down.
   while (reader_running &&
-	 reader_dispatching)
+	 reader_dispatching &&
+	 !reader_thread.am_self())
     cond.Wait(pipe_lock);
 }
 
@@ -1694,7 +1696,7 @@ void Pipe::writer()
       Message *m = _get_next_outgoing();
       if (m) {
 	m->set_seq(++out_seq);
-	if (!policy.lossy || close_on_empty) {
+	if (!policy.lossy) {
 	  // put on sent list
 	  sent.push_back(m); 
 	  m->get();
@@ -1755,12 +1757,6 @@ void Pipe::writer()
       continue;
     }
     
-    if (sent.empty() && close_on_empty) {
-      ldout(msgr->cct,10) << "writer out and sent queues empty, closing" << dendl;
-      stop();
-      continue;
-    }
-
     // wait
     ldout(msgr->cct,20) << "writer sleeping" << dendl;
     cond.Wait(pipe_lock);
