@@ -39,20 +39,19 @@ static ostream& _prefix(std::ostream *_dout, SimpleMessenger *msgr) {
 
 SimpleMessenger::SimpleMessenger(CephContext *cct, entity_name_t name,
 				 string mname, uint64_t _nonce)
-  : Messenger(cct, name),
+  : SimplePolicyMessenger(cct, name,mname, _nonce),
     accepter(this, _nonce),
     dispatch_queue(cct, this),
     reaper_thread(this),
-    my_type(name.type()),
     nonce(_nonce),
     lock("SimpleMessenger::lock"), need_addr(true), did_bind(false),
     global_seq(0),
     cluster_protocol(0),
-    policy_lock("SimpleMessenger::policy_lock"),
-    dispatch_throttler(cct, string("msgr_dispatch_throttler-") + mname, cct->_conf->ms_dispatch_throttle_bytes),
+    dispatch_throttler(cct, string("msgr_dispatch_throttler-") + mname,
+		       cct->_conf->ms_dispatch_throttle_bytes),
     reaper_started(false), reaper_stop(false),
     timeout(0),
-    local_connection(new Connection(cct, this))
+    local_connection(new PipeConnection(cct, this))
 {
   ceph_spin_init(&global_seq_lock);
   init_local_connection();
@@ -92,22 +91,21 @@ int SimpleMessenger::shutdown()
   return 0;
 }
 
-int SimpleMessenger::_send_message(Message *m, const entity_inst_t& dest,
-                                   bool lazy)
+int SimpleMessenger::_send_message(Message *m, const entity_inst_t& dest)
 {
   // set envelope
   m->get_header().src = get_myname();
 
   if (!m->get_priority()) m->set_priority(get_default_send_priority());
  
-  ldout(cct,1) << (lazy ? "lazy " : "") <<"--> " << dest.name << " "
+  ldout(cct,1) <<"--> " << dest.name << " "
           << dest.addr << " -- " << *m
     	  << " -- ?+" << m->get_data().length()
 	  << " " << m 
 	  << dendl;
 
   if (dest.addr == entity_addr_t()) {
-    ldout(cct,0) << (lazy ? "lazy_" : "") << "send_message message " << *m
+    ldout(cct,0) << "send_message message " << *m
                  << " with empty dest " << dest.addr << dendl;
     m->put();
     return -EINVAL;
@@ -116,25 +114,26 @@ int SimpleMessenger::_send_message(Message *m, const entity_inst_t& dest,
   lock.Lock();
   Pipe *pipe = _lookup_pipe(dest.addr);
   submit_message(m, (pipe ? pipe->connection_state.get() : NULL),
-                 dest.addr, dest.name.type(), lazy, true);
+                 dest.addr, dest.name.type(), true);
   lock.Unlock();
   return 0;
 }
 
-int SimpleMessenger::_send_message(Message *m, Connection *con, bool lazy)
+int SimpleMessenger::_send_message(Message *m, Connection *con)
 {
   //set envelope
   m->get_header().src = get_myname();
 
   if (!m->get_priority()) m->set_priority(get_default_send_priority());
 
-  ldout(cct,1) << (lazy ? "lazy " : "") << "--> " << con->get_peer_addr()
+  ldout(cct,1) << "--> " << con->get_peer_addr()
       << " -- " << *m
       << " -- ?+" << m->get_data().length()
       << " " << m << " con " << con
       << dendl;
 
-  submit_message(m, con, con->get_peer_addr(), con->get_peer_type(), lazy, false);
+  submit_message(m, static_cast<PipeConnection*>(con),
+		 con->get_peer_addr(), con->get_peer_type(), false);
   return 0;
 }
 
@@ -154,6 +153,8 @@ void SimpleMessenger::set_addr_unknowns(entity_addr_t &addr)
 
 int SimpleMessenger::get_proto_version(int peer_type, bool connect)
 {
+  int my_type = my_inst.name.type();
+
   // set reply protocol version
   if (peer_type == my_type) {
     // internal
@@ -224,7 +225,8 @@ void SimpleMessenger::reaper()
   while (!pipe_reap_queue.empty()) {
     Pipe *p = pipe_reap_queue.front();
     pipe_reap_queue.pop_front();
-    ldout(cct,10) << "reaper reaping pipe " << p << " " << p->get_peer_addr() << dendl;
+    ldout(cct,10) << "reaper reaping pipe " << p << " " <<
+      p->get_peer_addr() << dendl;
     p->pipe_lock.Lock();
     p->discard_out_queue();
     if (p->connection_state) {
@@ -300,7 +302,7 @@ int SimpleMessenger::start()
   ldout(cct,1) << "messenger.start" << dendl;
 
   // register at least one entity, first!
-  assert(my_type >= 0);
+  assert(my_inst.name.type() >= 0);
 
   assert(!started);
   started = true;
@@ -336,7 +338,7 @@ Pipe *SimpleMessenger::add_accept_pipe(int sd)
  */
 Pipe *SimpleMessenger::connect_rank(const entity_addr_t& addr,
 				    int type,
-				    Connection *con,
+				    PipeConnection *con,
 				    Message *first)
 {
   assert(lock.is_locked());
@@ -345,7 +347,8 @@ Pipe *SimpleMessenger::connect_rank(const entity_addr_t& addr,
   ldout(cct,10) << "connect_rank to " << addr << ", creating pipe and registering" << dendl;
   
   // create pipe
-  Pipe *pipe = new Pipe(this, Pipe::STATE_CONNECTING, con);
+  Pipe *pipe = new Pipe(this, Pipe::STATE_CONNECTING,
+			static_cast<PipeConnection*>(con));
   pipe->pipe_lock.Lock();
   pipe->set_peer_type(type);
   pipe->set_peer_addr(addr);
@@ -406,9 +409,9 @@ ConnectionRef SimpleMessenger::get_loopback_connection()
   return local_connection;
 }
 
-void SimpleMessenger::submit_message(Message *m, Connection *con,
+void SimpleMessenger::submit_message(Message *m, PipeConnection *con,
 				     const entity_addr_t& dest_addr, int dest_type,
-				     bool lazy, bool already_locked)
+				     bool already_locked)
 {
   if (cct->_conf->ms_dump_on_send) {
     m->encode(-1, true);
@@ -425,7 +428,7 @@ void SimpleMessenger::submit_message(Message *m, Connection *con,
   // existing connection?
   if (con) {
     Pipe *pipe = NULL;
-    bool ok = con->try_get_pipe((RefCountedObject**)&pipe);
+    bool ok = static_cast<PipeConnection*>(con)->try_get_pipe(&pipe);
     if (!ok) {
       ldout(cct,0) << "submit_message " << *m << " remote, " << dest_addr
 		   << ", failed lossy con, dropping message " << m << dendl;
@@ -443,7 +446,7 @@ void SimpleMessenger::submit_message(Message *m, Connection *con,
 	return;
       }
       Pipe *current_pipe;
-      ok = con->try_get_pipe((RefCountedObject**)&current_pipe);
+      ok = con->try_get_pipe(&current_pipe);
       pipe->pipe_lock.Unlock();
       if (current_pipe == pipe) {
 	ldout(cct,20) << "submit_message " << *m << " remote, " << dest_addr
@@ -473,9 +476,6 @@ void SimpleMessenger::submit_message(Message *m, Connection *con,
     ldout(cct,20) << "submit_message " << *m << " remote, " << dest_addr << ", lossy server for target type "
 		  << ceph_entity_type_name(dest_type) << ", no session, dropping." << dendl;
     m->put();
-  } else if (lazy) {
-    ldout(cct,20) << "submit_message " << *m << " remote, " << dest_addr << ", lazy, dropping." << dendl;
-    m->put();
   } else {
     ldout(cct,20) << "submit_message " << *m << " remote, " << dest_addr << ", new pipe." << dendl;
     if (!already_locked) {
@@ -483,47 +483,18 @@ void SimpleMessenger::submit_message(Message *m, Connection *con,
        *  grab the lock and do it again. If we got here, we know it's a non-lossy
        *  Connection, so we can use our existing pointer without doing another lookup. */
       Mutex::Locker l(lock);
-      submit_message(m, con, dest_addr, dest_type, lazy, true);
+      submit_message(m, con, dest_addr, dest_type, true);
     } else {
-      connect_rank(dest_addr, dest_type, con, m);
+      connect_rank(dest_addr, dest_type, static_cast<PipeConnection*>(con), m);
     }
   }
 }
 
-int SimpleMessenger::send_keepalive(const entity_inst_t& dest)
-{
-  const entity_addr_t dest_addr = dest.addr;
-  entity_addr_t dest_proc_addr = dest_addr;
-  int ret = 0;
-
-  lock.Lock();
-  {
-    // local?
-    if (my_inst.addr != dest_addr) {
-      // remote.
-      Pipe *pipe = _lookup_pipe(dest_proc_addr);
-      if (pipe) {
-        // connected?
-	pipe->pipe_lock.Lock();
-	ldout(cct,20) << "send_keepalive remote, " << dest_addr << ", have pipe." << dendl;
-	pipe->_send_keepalive();
-	pipe->pipe_lock.Unlock();
-      } else {
-        ret = -EINVAL;
-      }
-      if (!pipe) {
-	ldout(cct,20) << "send_keepalive no pipe for " << dest_addr << ", doing nothing." << dendl;
-      }
-    }
-  }
-  lock.Unlock();
-  return ret;
-}
-
 int SimpleMessenger::send_keepalive(Connection *con)
 {
   int ret = 0;
-  Pipe *pipe = static_cast<Pipe *>(con->get_pipe());
+  Pipe *pipe = static_cast<Pipe *>(
+    static_cast<PipeConnection*>(con)->get_pipe());
   if (pipe) {
     ldout(cct,20) << "send_keepalive con " << con << ", have pipe." << dendl;
     assert(pipe->msgr == this);
@@ -599,7 +570,6 @@ void SimpleMessenger::wait()
   ldout(cct,10) << "wait: done." << dendl;
   ldout(cct,1) << "shutdown complete." << dendl;
   started = false;
-  my_type = -1;
 }
 
 
@@ -612,7 +582,7 @@ void SimpleMessenger::mark_down_all()
     ldout(cct,5) << "mark_down_all accepting_pipe " << p << dendl;
     p->pipe_lock.Lock();
     p->stop_and_wait();
-    ConnectionRef con = p->connection_state;
+    PipeConnectionRef con = p->connection_state;
     if (con && con->clear_pipe(p))
       dispatch_queue.queue_reset(con.get());
     p->pipe_lock.Unlock();
@@ -627,7 +597,7 @@ void SimpleMessenger::mark_down_all()
     p->unregister_pipe();
     p->pipe_lock.Lock();
     p->stop_and_wait();
-    ConnectionRef con = p->connection_state;
+    PipeConnectionRef con = p->connection_state;
     if (con && con->clear_pipe(p))
       dispatch_queue.queue_reset(con.get());
     p->pipe_lock.Unlock();
@@ -648,7 +618,7 @@ void SimpleMessenger::mark_down(const entity_addr_t& addr)
       // generate a reset event for the caller in this case, even
       // though they asked for it, since this is the addr-based (and
       // not Connection* based) interface
-      ConnectionRef con = p->connection_state;
+      PipeConnectionRef con = p->connection_state;
       if (con && con->clear_pipe(p))
 	dispatch_queue.queue_reset(con.get());
     }
@@ -664,7 +634,7 @@ void SimpleMessenger::mark_down(Connection *con)
   if (con == NULL)
     return;
   lock.Lock();
-  Pipe *p = static_cast<Pipe *>(con->get_pipe());
+  Pipe *p = static_cast<Pipe *>(static_cast<PipeConnection*>(con)->get_pipe());
   if (p) {
     ldout(cct,1) << "mark_down " << con << " -- " << p << dendl;
     assert(p->msgr == this);
@@ -684,33 +654,10 @@ void SimpleMessenger::mark_down(Connection *con)
   lock.Unlock();
 }
 
-void SimpleMessenger::mark_down_on_empty(Connection *con)
-{
-  lock.Lock();
-  Pipe *p = static_cast<Pipe *>(con->get_pipe());
-  if (p) {
-    assert(p->msgr == this);
-    p->pipe_lock.Lock();
-    p->unregister_pipe();
-    if (p->out_q.empty()) {
-      ldout(cct,1) << "mark_down_on_empty " << con << " -- " << p << " closing (queue is empty)" << dendl;
-      p->stop_and_wait();
-    } else {
-      ldout(cct,1) << "mark_down_on_empty " << con << " -- " << p << " marking (queue is not empty)" << dendl;
-      p->close_on_empty = true;
-    }
-    p->pipe_lock.Unlock();
-    p->put();
-  } else {
-    ldout(cct,1) << "mark_down_on_empty " << con << " -- pipe dne" << dendl;
-  }
-  lock.Unlock();
-}
-
 void SimpleMessenger::mark_disposable(Connection *con)
 {
   lock.Lock();
-  Pipe *p = static_cast<Pipe *>(con->get_pipe());
+  Pipe *p = static_cast<Pipe *>(static_cast<PipeConnection*>(con)->get_pipe());
   if (p) {
     ldout(cct,1) << "mark_disposable " << con << " -- " << p << dendl;
     assert(p->msgr == this);
@@ -757,6 +704,6 @@ void SimpleMessenger::unlearn_addr()
 void SimpleMessenger::init_local_connection()
 {
   local_connection->peer_addr = my_inst.addr;
-  local_connection->peer_type = my_type;
+  local_connection->peer_type = my_inst.name.type();
   ms_deliver_handle_fast_connect(local_connection.get());
 }
