@@ -24,11 +24,10 @@
 ObjectStore *ObjectStore::create(CephContext *cct,
 				 const string& type,
 				 const string& data,
-				 const string& journal,
-			         osflagbits_t flags)
+				 const string& journal)
 {
   if (type == "filestore") {
-    return new FileStore(data, journal, flags);
+    return new FileStore(data, journal);
   }
   if (type == "memstore") {
     return new MemStore(cct, data);
@@ -113,6 +112,7 @@ int ObjectStore::queue_transactions(
 			    onreadable_sync, op);
 }
 
+
 int ObjectStore::collection_list(coll_t c, vector<hobject_t>& o)
 {
   vector<ghobject_t> go;
