@@ -128,6 +128,22 @@ public:
    */
   struct Sequencer_impl {
     virtual void flush() = 0;
+
+    /**
+     * Async flush_commit
+     *
+     * There are two cases:
+     * 1) sequencer is currently idle: the method returns true and
+     *    c is deleted
+     * 2) sequencer is not idle: the method returns false and c is
+     *    called asyncronously with a value of 0 once all transactions
+     *    queued on this sequencer prior to the call have been applied
+     *    and committed.
+     */
+    virtual bool flush_commit(
+      Context *c ///< [in] context to call upon flush/commit
+      ) = 0; ///< @return true if idle, false otherwise
+
     virtual ~Sequencer_impl() {}
   };
 
@@ -153,6 +169,16 @@ public:
       if (p)
 	p->flush();
     }
+
+    /// @see Sequencer_impl::flush_commit()
+    bool flush_commit(Context *c) {
+      if (!p) {
+	delete c;
+	return true;
+      } else {
+	return p->flush_commit(c);
+      }
+    }
   };
 
   /*********************************
@@ -537,16 +563,16 @@ public:
        * stream. There is no checking that the encoded data is of the
        * correct type.
        */
-      int get_op() {
+      int decode_op() {
 	__u32 op;
 	::decode(op, p);
 	return op;
       }
-      void get_bl(bufferlist& bl) {
+      void decode_bl(bufferlist& bl) {
 	::decode(bl, p);
       }
       /// Get an oid, recognize various legacy forms and update them.
-      ghobject_t get_oid() {
+      ghobject_t decode_oid() {
 	ghobject_t oid;
 	if (sobject_encoding) {
 	  sobject_t soid;
@@ -554,7 +580,7 @@ public:
 	  oid.hobj.snap = soid.snap;
 	  oid.hobj.oid = soid.oid;
 	  oid.generation = ghobject_t::NO_GEN;
-	  oid.shard_id = ghobject_t::NO_SHARD;
+	  oid.shard_id = shard_id_t::NO_SHARD;
 	} else {
 	  ::decode(oid, p);
 	  if (use_pool_override && pool_override != -1 &&
@@ -564,36 +590,36 @@ public:
 	}
 	return oid;
       }
-      coll_t get_cid() {
+      coll_t decode_cid() {
 	coll_t c;
 	::decode(c, p);
 	return c;
       }
-      uint64_t get_length() {
+      uint64_t decode_length() {
 	uint64_t len;
 	::decode(len, p);
 	return len;
       }
-      string get_attrname() {
+      string decode_attrname() {
 	string s;
 	::decode(s, p);
 	return s;
       }
-      string get_key() {
+      string decode_key() {
 	string s;
 	::decode(s, p);
 	return s;
       }
-      void get_attrset(map<string,bufferptr>& aset) {
+      void decode_attrset(map<string,bufferptr>& aset) {
 	::decode(aset, p);
       }
-      void get_attrset(map<string,bufferlist>& aset) {
+      void decode_attrset(map<string,bufferlist>& aset) {
 	::decode(aset, p);
       }
-      void get_keyset(set<string> &keys) {
+      void decode_keyset(set<string> &keys) {
 	::decode(keys, p);
       }
-      uint32_t get_u32() {
+      uint32_t decode_u32() {
 	uint32_t bits;
 	::decode(bits, p);
 	return bits;
@@ -1166,7 +1192,8 @@ public:
   virtual bool test_mount_in_use() = 0;
   virtual int mount() = 0;
   virtual int umount() = 0;
-  virtual int get_max_object_name_length() = 0;
+  virtual unsigned get_max_object_name_length() = 0;
+  virtual unsigned get_max_attr_name_length() = 0;
   virtual int mkfs() = 0;  // wipe
   virtual int mkjournal() = 0; // journal only
   virtual void set_allow_sharded_objects() = 0;
@@ -1342,10 +1369,9 @@ public:
    * @param cid collection for object
    * @param oid oid of object
    * @param aset place to put output result.
-   * @param user_only true -> only user attributes are return else all attributes are returned
    * @returns 0 on success, negative error code on failure.
    */
-  virtual int getattrs(coll_t cid, const ghobject_t& oid, map<string,bufferptr>& aset, bool user_only = false) = 0;
+  virtual int getattrs(coll_t cid, const ghobject_t& oid, map<string,bufferptr>& aset) = 0;
 
   /**
    * getattrs -- get all of the xattrs of an object
@@ -1353,12 +1379,11 @@ public:
    * @param cid collection for object
    * @param oid oid of object
    * @param aset place to put output result.
-   * @param user_only true -> only user attributes are return else all attributes are returned
    * @returns 0 on success, negative error code on failure.
    */
-  int getattrs(coll_t cid, const ghobject_t& oid, map<string,bufferlist>& aset, bool user_only = false) {
+  int getattrs(coll_t cid, const ghobject_t& oid, map<string,bufferlist>& aset) {
     map<string,bufferptr> bmap;
-    int r = getattrs(cid, oid, bmap, user_only);
+    int r = getattrs(cid, oid, bmap);
     for (map<string,bufferptr>::iterator i = bmap.begin();
 	i != bmap.end();
 	++i) {
