@@ -3,7 +3,8 @@
 
 #include "chain_xattr.h"
 
-#include <inttypes.h>
+#include "include/int_types.h"
+
 #include <unistd.h>
 #include <stdlib.h>
 #include <sys/types.h>
@@ -21,10 +22,6 @@
 #include <linux/fs.h>
 #endif
 
-#if defined(__FreeBSD__)
-#include "include/inttypes.h"
-#endif
-
 #include "common/xattr.h"
 
 /*
@@ -41,7 +38,6 @@
 
 static void get_raw_xattr_name(const char *name, int i, char *raw_name, int raw_len)
 {
-  int r;
   int pos = 0;
 
   while (*name) {
@@ -66,7 +62,7 @@ static void get_raw_xattr_name(const char *name, int i, char *raw_name, int raw_
   if (!i) {
     *raw_name = '\0';
   } else {
-    r = snprintf(raw_name, raw_len, "@%d", i);
+    int r = snprintf(raw_name, raw_len, "@%d", i);
     assert(r < raw_len - pos);
   }
 }
@@ -256,13 +252,15 @@ int chain_setxattr(const char *fn, const char *name, const void *val, size_t siz
     i++;
   } while (size);
 
-  /* if we're exactly at a chunk size, remove the next one (if wasn't removed
-     before) */
-  if (ret >= 0 && chunk_size == CHAIN_XATTR_MAX_BLOCK_LEN) {
-    get_raw_xattr_name(name, i, raw_name, sizeof(raw_name));
-    int r = sys_removexattr(fn, raw_name);
-    if (r < 0 && r != -ENODATA)
-      ret = r;
+  if (ret >= 0 ) {
+    int r;
+    do {
+      get_raw_xattr_name(name, i, raw_name, sizeof(raw_name));
+      r = sys_removexattr(fn, raw_name);
+      if (r < 0 && r != -ENODATA)
+	ret = r;
+      i++;
+    } while (r != -ENODATA);
   }
   
   return ret;
@@ -290,13 +288,15 @@ int chain_fsetxattr(int fd, const char *name, const void *val, size_t size)
     i++;
   } while (size);
 
-  /* if we're exactly at a chunk size, remove the next one (if wasn't removed
-     before) */
-  if (ret >= 0 && chunk_size == CHAIN_XATTR_MAX_BLOCK_LEN) {
-    get_raw_xattr_name(name, i, raw_name, sizeof(raw_name));
-    int r = sys_fremovexattr(fd, raw_name);
-    if (r < 0 && r != -ENODATA)
-      ret = r;
+  if (ret >= 0) {
+    int r;
+    do {
+      get_raw_xattr_name(name, i, raw_name, sizeof(raw_name));
+      r = sys_fremovexattr(fd, raw_name);
+      if (r < 0 && r != -ENODATA)
+	ret = r;
+      i++;
+    } while (r != -ENODATA);
   }
   
   return ret;
@@ -358,8 +358,10 @@ int chain_listxattr(const char *fn, char *names, size_t len) {
     return -ENOMEM;
 
   r = sys_listxattr(fn, full_buf, total_len);
-  if (r < 0)
+  if (r < 0) {
+    free(full_buf);
     return r;
+  }
 
   char *p = full_buf;
   const char *end = full_buf + r;
@@ -390,6 +392,10 @@ done:
 
 int chain_flistxattr(int fd, char *names, size_t len) {
   int r;
+  char *p;
+  const char * end;
+  char *dest;
+  char *dest_end;
 
   if (!len)
     return sys_flistxattr(fd, names, len) * 2;
@@ -405,12 +411,12 @@ int chain_flistxattr(int fd, char *names, size_t len) {
 
   r = sys_flistxattr(fd, full_buf, total_len);
   if (r < 0)
-    return r;
+    goto done;
 
-  char *p = full_buf;
-  const char *end = full_buf + r;
-  char *dest = names;
-  char *dest_end = names + len;
+  p = full_buf;
+  end = full_buf + r;
+  dest = names;
+  dest_end = names + len;
 
   while (p < end) {
     char name[CHAIN_XATTR_MAX_NAME_LEN * 2 + 16];
