@@ -97,6 +97,7 @@ public:
     const eversion_t &at_version,
     PGTransaction *t,
     const eversion_t &trim_to,
+    const eversion_t &trim_rollback_to,
     vector<pg_log_entry_t> &log_entries,
     boost::optional<pg_hit_set_history_t> &hset_history,
     Context *on_local_applied_sync,
@@ -148,10 +149,8 @@ public:
 private:
   friend struct ECRecoveryHandle;
   uint64_t get_recovery_chunk_size() const {
-    uint64_t max = cct->_conf->osd_recovery_max_chunk;
-    max -= max % sinfo.get_stripe_width();
-    max += sinfo.get_stripe_width();
-    return max;
+    return ROUND_UP_TO(cct->_conf->osd_recovery_max_chunk,
+			sinfo.get_stripe_width());
   }
 
   /**
@@ -326,6 +325,7 @@ public:
     hobject_t hoid;
     eversion_t version;
     eversion_t trim_to;
+    eversion_t trim_rollback_to;
     vector<pg_log_entry_t> log_entries;
     boost::optional<pg_hit_set_history_t> updated_hit_set_history;
     Context *on_local_applied_sync;
