@@ -477,6 +477,8 @@ public:
   eversion_t  min_last_complete_ondisk;  // up: min over last_complete_ondisk, peer_last_complete_ondisk
   eversion_t  pg_trim_to;
 
+  set<int> blocked_by; ///< osds we are blocked by (for pg stats)
+
   // [primary only] content recovery state
  protected:
   struct PriorSet {
@@ -749,6 +751,7 @@ protected:
   ceph::shared_ptr<ObjectStore::Sequencer> osr;
 
   void _update_calc_stats();
+  void _update_blocked_by();
   void publish_stats_to_osd();
   void clear_publish_stats();
 
@@ -1637,8 +1640,8 @@ public:
       Active(my_context ctx);
       void exit();
 
-      const set<pg_shard_t> sorted_actingbackfill_set;
-      const set<pg_shard_t> sorted_backfill_set;
+      const set<pg_shard_t> remote_shards_to_reserve_recovery;
+      const set<pg_shard_t> remote_shards_to_reserve_backfill;
       bool all_replicas_activated;
 
       typedef boost::mpl::list <
@@ -1803,7 +1806,7 @@ public:
 	boost::statechart::custom_reaction< RemoteRecoveryReserved >,
 	boost::statechart::transition< AllRemotesReserved, Recovering >
 	> reactions;
-      set<pg_shard_t>::const_iterator acting_osd_it;
+      set<pg_shard_t>::const_iterator remote_recovery_reservation_it;
       WaitRemoteRecoveryReserved(my_context ctx);
       boost::statechart::result react(const RemoteRecoveryReserved &evt);
       void exit();
@@ -2051,6 +2054,7 @@ public:
   bool       is_replay() const { return state_test(PG_STATE_REPLAY); }
   bool       is_clean() const { return state_test(PG_STATE_CLEAN); }
   bool       is_degraded() const { return state_test(PG_STATE_DEGRADED); }
+  bool       is_undersized() const { return state_test(PG_STATE_UNDERSIZED); }
 
   bool       is_scrubbing() const { return state_test(PG_STATE_SCRUBBING); }
 
