@@ -1,4 +1,5 @@
-
+// -*- mode:C++; tab-width:8; c-basic-offset:2; indent-tabs-mode:t -*-
+// vim: ts=8 sw=2 smarttab
 
 #include "rgw_fcgi.h"
 
@@ -32,7 +33,7 @@ void RGWFCGX::init_env(CephContext *cct)
 
 int RGWFCGX::send_status(const char *status, const char *status_name)
 {
-  return print("Status: %s %s\r\n", status, status_name);
+  return print("Status: %s\n", status);
 }
 
 int RGWFCGX::send_100_continue()
@@ -47,8 +48,8 @@ int RGWFCGX::send_100_continue()
 int RGWFCGX::send_content_length(uint64_t len)
 {
   char buf[21];
-  snprintf(buf, sizeof(buf), "%"PRIu64, len);
-  return print("Content-Length: %s\r\n", buf);
+  snprintf(buf, sizeof(buf), "%" PRIu64, len);
+  return print("Content-Length: %s\n", buf);
 }
 
 int RGWFCGX::complete_header()
