@@ -1,3 +1,6 @@
+// -*- mode:C++; tab-width:8; c-basic-offset:2; indent-tabs-mode:t -*-
+// vim: ts=8 sw=2 smarttab
+
 #include <errno.h>
 
 #include "common/errno.h"
@@ -10,11 +13,12 @@
 
 #define dout_subsys ceph_subsys_rgw
 
-#define READ_CHUNK_LEN (16 * 1024)
+#define READ_CHUNK_LEN (512 * 1024)
 
 static map<string, string> ext_mime_map;
 
-int rgw_put_system_obj(RGWRados *rgwstore, rgw_bucket& bucket, string& oid, const char *data, size_t size, bool exclusive, map<string, bufferlist> *pattrs)
+int rgw_put_system_obj(RGWRados *rgwstore, rgw_bucket& bucket, string& oid, const char *data, size_t size, bool exclusive,
+                       RGWObjVersionTracker *objv_tracker, time_t set_mtime, map<string, bufferlist> *pattrs)
 {
   map<string,bufferlist> no_attrs;
   if (!pattrs)
@@ -22,44 +26,46 @@ int rgw_put_system_obj(RGWRados *rgwstore, rgw_bucket& bucket, string& oid, cons
 
   rgw_obj obj(bucket, oid);
 
-  int ret = rgwstore->put_obj(NULL, obj, data, size, exclusive, NULL, *pattrs);
+  int ret = rgwstore->put_system_obj(NULL, obj, data, size, exclusive, NULL, *pattrs, objv_tracker, set_mtime);
 
   if (ret == -ENOENT) {
     ret = rgwstore->create_pool(bucket);
     if (ret >= 0)
-      ret = rgwstore->put_obj(NULL, obj, data, size, exclusive, NULL, *pattrs);
+      ret = rgwstore->put_system_obj(NULL, obj, data, size, exclusive, NULL, *pattrs, objv_tracker, set_mtime);
   }
 
   return ret;
 }
 
-int rgw_get_obj(RGWRados *rgwstore, void *ctx, rgw_bucket& bucket, string& key, bufferlist& bl, map<string, bufferlist> *pattrs)
+int rgw_get_system_obj(RGWRados *rgwstore, void *ctx, rgw_bucket& bucket, const string& key, bufferlist& bl,
+                       RGWObjVersionTracker *objv_tracker, time_t *pmtime, map<string, bufferlist> *pattrs,
+                       rgw_cache_entry_info *cache_info)
 {
-  int ret;
   struct rgw_err err;
   void *handle = NULL;
   bufferlist::iterator iter;
   int request_len = READ_CHUNK_LEN;
   rgw_obj obj(bucket, key);
-  ret = rgwstore->prepare_get_obj(ctx, obj, NULL, NULL, pattrs, NULL,
-                                  NULL, NULL, NULL, NULL, NULL, NULL, &handle, &err);
-  if (ret < 0)
-    return ret;
 
   do {
-    ret = rgwstore->get_obj(ctx, &handle, obj, bl, 0, request_len - 1);
+    int ret = rgwstore->prepare_get_obj(ctx, obj, NULL, NULL, pattrs, NULL,
+                                        NULL, pmtime, NULL, NULL, NULL, NULL,
+                                        objv_tracker, &handle, &err);
     if (ret < 0)
-      goto done;
+      return ret;
+
+    ret = rgwstore->get_obj(ctx, objv_tracker, &handle, obj, bl, 0, request_len - 1, cache_info);
+    rgwstore->finish_get_obj(&handle);
+    if (ret < 0)
+      return ret;
+
     if (ret < request_len)
       break;
     bl.clear();
     request_len *= 2;
   } while (true);
 
-  ret = 0;
-done:
-  rgwstore->finish_get_obj(&handle);
-  return ret;
+  return 0;
 }
 
 void parse_mime_map_line(const char *start, const char *end)
@@ -161,3 +167,8 @@ int rgw_tools_init(CephContext *cct)
 
   return 0;
 }
+
+void rgw_tools_cleanup()
+{
+  ext_mime_map.clear();
+}
