@@ -24,7 +24,7 @@
 #include <stdint.h>
 #include <sys/stat.h>
 #include <sys/types.h>
-#include <tr1/memory>
+#include "include/memory.h"
 
 using ceph::bufferlist;
 using std::cerr;
@@ -78,7 +78,7 @@ static int create_tempfile(const std::string &fname, const char *text)
 	 << get_temp_dir() << "'. " << cpp_strerror(err) << std::endl;
     return err;
   }
-  std::tr1::shared_ptr<FILE> fpp(fp, fclose);
+  ceph::shared_ptr<FILE> fpp(fp, fclose);
   if (unlink_idx >= MAX_FILES_TO_DELETE)
     return -ENOBUFS;
   if (unlink_idx == 0) {
@@ -417,7 +417,6 @@ TEST(ConfUtils, IllegalFiles) {
   std::ostringstream warn;
   std::string illegal_conf1_f(next_tempfile(illegal_conf1));
   ConfFile cf1;
-  std::string val;
   ASSERT_EQ(cf1.parse_file(illegal_conf1_f.c_str(), &err, &warn), 0);
   ASSERT_EQ(err.size(), 1U);
 
