@@ -30,6 +30,7 @@ function run_osd() {
 
     touch $dir/ceph.conf
 
+    mkdir -p $osd_data
     ./ceph-disk $ceph_disk_args \
         prepare $osd_data || return 1
 
@@ -37,13 +38,13 @@ function run_osd() {
     ceph_args+=" --osd-journal-size=100"
     ceph_args+=" --osd-data=$osd_data"
     ceph_args+=" --chdir="
-    ceph_args+=" --osd-pool-default-erasure-code-directory=.libs"
     ceph_args+=" --run-dir=$dir"
     ceph_args+=" --debug-osd=20"
     ceph_args+=" --log-file=$dir/osd-\$id.log"
     ceph_args+=" --pid-file=$dir/osd-\$id.pidfile"
     ceph_args+=" "
     ceph_args+="$@"
+    mkdir -p $osd_data
     CEPH_ARGS="$ceph_args" ./ceph-disk $ceph_disk_args \
         activate \
         --mark-init=none \
@@ -52,4 +53,17 @@ function run_osd() {
     [ "$id" = "$(cat $osd_data/whoami)" ] || return 1
 
     ./ceph osd crush create-or-move "$id" 1 root=default host=localhost
+
+    status=1
+    # Workaround for http://tracker.ceph.com/issues/8630
+    for ((i=0; i < 60; i++)); do
+        if ! ceph osd dump | grep "osd.$id up"; then
+            sleep 1
+        else
+            status=0
+            break
+        fi
+    done
+
+    return $status
 }
